/*
 * UniversityDirectory.java
 *
 * Created on October 22, 2004, 5:16 AM
 */

package university;

import java.util.*;
import java.io.Serializable;
/**
 * CategoryDirectory containing the people of the university.
 *
 * @author  Nathan Balon
 * University Application
 * CIS 550 - Object Oriented Programming
 * U 0f M Dearborn
 * November 7, 2004
 * @version 1.0
 */

public class CategoryDirectory implements Directory, Serializable, Cloneable{
    
    /** Creates a new instance of UniversityDirectory */
    public CategoryDirectory() {
        undergraduateList = new ArrayList<Undergraduate>();
        graduateList = new ArrayList<Graduate>();
        facultyList = new ArrayList<Faculty>();
        staffList = new ArrayList <Staff>();
        idCategoryLookup = new HashMap<Integer, String>();
        nameCategoryLookup = new HashMap <String, String>();        
    }
    
    /**
     * Clone a new object
     * @throws java.lang.CloneNotSupportedException Throws CloneNotSupportedException
     * @return Object a cloned object
     */    
    @Override
    public Object clone() throws java.lang.CloneNotSupportedException{
        CategoryDirectory cloned = (CategoryDirectory)super.clone();
        for(Undergraduate student : undergraduateList){
            cloned.undergraduateList.add((Undergraduate)student.clone());
        }
        for(Graduate student : graduateList){
            cloned.graduateList.add((Graduate)student.clone());
        }
        for(Faculty employee : facultyList){
            cloned.facultyList.add((Faculty)employee.clone());
        }
        for(Staff employee : staffList){
            cloned.staffList.add((Staff)employee.clone());
        }
        cloned.idCategoryLookup = (HashMap)idCategoryLookup.clone();
        cloned.nameCategoryLookup = (HashMap)nameCategoryLookup.clone();
        return cloned;
    }    
    
    /**
     *  Add a Person to the UniversityDirectory 
     *  @param person The person to add to the UniversityDirectory
     */
    public void add(Person person){
        String name = person.getName();
        int id = person.getID();
        // if person is a graduate add to graduate list       
        if(person instanceof Graduate){
            idCategoryLookup.put(id, "graduate");
            nameCategoryLookup.put(name, "graduate");             
            graduateList.add((Graduate)person);
        }
        // if person is an undergraduate add to the undergraduate list
        else if(person instanceof Undergraduate){
            idCategoryLookup.put(id, "undergraduate");
            nameCategoryLookup.put(name, "undergraduate");
            undergraduateList.add((Undergraduate)person);
        }
        // if person is a faculty member add to the faculty list
        else if(person instanceof Faculty){
            idCategoryLookup.put(id, "faculty");
            nameCategoryLookup.put(name, "faculty");
            facultyList.add((Faculty)person);           
        }
        // else if the person is a staff member add to the staff list
        else{
            idCategoryLookup.put(id, "staff");
            nameCategoryLookup.put(name, "staff");            
            staffList.add((Staff)person);
        }
    }
    
    /** 
     *  Remove a Person from the UniversityDirectory
     *  @param person The person to remove from the UniversityDirectory
     */
    public void remove(Person person){
        // remove the person from both of the lookup table they are contained 
        // in and remove them from the list they are contained in.
        String name = person.getName();
        int id = person.getID();
        idCategoryLookup.remove(id);
        nameCategoryLookup.remove(name);        
        
        if(person instanceof Graduate){  
            for(int i =  0; i < graduateList.size(); i++){
                if(graduateList.get(i).getID() == id){
                    graduateList.remove(i);
                }
            }            
        }
        else if(person instanceof Undergraduate){
            for(int i =  0; i < undergraduateList.size(); i++){
                if(undergraduateList.get(i).getID() == id){
                    undergraduateList.remove(i);
                }
            }
        }
        else if(person instanceof Faculty){
            for(int i =  0; i < facultyList.size(); i++){
                if(facultyList.get(i).getID() == id){
                    facultyList.remove(i);
                }
            }                   
        }
        else{           
            for(int i =  0; i < staffList.size(); i++){
                if(staffList.get(i).getID() == id){
                   staffList.remove(i);
                }
            }
        }        
    }

    /** List all people in the directory */
    public String listAll(){
        // creates a string builder and add all the people from each list to 
        // the directory list.
        StringBuilder builder = new StringBuilder();
        for(int i = 0; i < undergraduateList.size() ; i++){
            builder.append(undergraduateList.get(i) + "\n");
        }
        for(int i = 0; i < graduateList.size() ; i++){
            builder.append(graduateList.get(i) + "\n");
        }            
        for(int i = 0; i < staffList.size() ; i++){
            builder.append(staffList.get(i) + "\n");
        } 
        for(int i = 0; i < facultyList.size() ; i++){
            builder.append(facultyList.get(i) + "\n");
        } 
        return builder.toString();
    }
    
    /**
     * List the size for each category member of the university.
     * @return String listing the number of people in each category.
     */    
    public String size(){
        return ("Undergraduate:             " + undergraduateList.size() +
                "\nGraduate:                  " + graduateList.size() +
                "\nFaculty                    " + facultyList.size() +
                "\nStaff                      " + staffList.size() + "\n");   
        
    }
    
    /**
     * Search for a Person by name.
     * @param name the name of the Person
     * @return the person
     */    
    public Person search(String name){
       // get the name of the list the person is contained in then 
       // search the list to find the person
       String category = nameCategoryLookup.get(name);
        if(category.equals("undergraduate")){
            for(Person p: undergraduateList){
                if(p.getName().equals(name)){
                    return p;
                }
            }
        }
        else if(category.equals("graduate")){
            for(Person p: graduateList){
                if(p.getName().equals(name)){
                    return p;
                }
            }
        }  
        else if(category.equals("faculty")){
            for(Person p: facultyList){
                if(p.getName().equals(name)){
                    return p;
                }
            }
        }   
        else{for(Person p: staffList){
                if(p.getName().equals(name)){
                    return p;
                }
            } 
        }
        return null;
    }
    /**
     * Search for a Person by ID.
     * @param id the ID number of the Person
     * @return the person
     */    
    public Person search(int id){
       // get the name of the list the person is contained in then 
       // search the list to find the person
        String category = idCategoryLookup.get(id);
        if(category.equals("undergraduate")){
            for(Person p: undergraduateList){
                if(p.getID() == id){
                    return p;
                }
            }
        }
        else if(category.equals("graduate")){
            for(Person p: graduateList){
                if(p.getID() == id){
                    return p;
                }
            }
        }  
        else if(category.equals("faculty")){
            for(Person p: facultyList){
                if(p.getID() == id){
                    return p;
                }
            }
        }   
        else{for(Person p: staffList){
                if(p.getID() == id){
                    return p;
                }
            } 
        }
        return null;
    }
    
    public String displayCategory(int categorySelection){
        String seperator = "-----------------------------------------------"+ 
                           "-----------------------------------------------\n";
        StringBuilder builder = new StringBuilder();
        if(categorySelection == 1){
            builder.append("\t\tFaculty\n");
            builder.append(seperator);
            for(int i = 0; i < staffList.size(); i++){
                builder.append(staffList.get(i) + "\n");              
            }
            return builder.toString();              
        }
        else if(categorySelection == 2){
            builder.append("\t\tStaff\n");
            builder.append(seperator);
            for(int i = 0; i < facultyList.size(); i++){
                builder.append(facultyList.get(i) + "\n");           
            }
            return builder.toString();     
        }
        else if(categorySelection == 3){
          builder.append("\t\tUndergraduate\n");
          builder.append(seperator);            
            for(int i = 0; i < undergraduateList.size(); i++){
                builder.append(undergraduateList.get(i) + "\n");                
            }
            return builder.toString();
        } 
        else if(categorySelection == 4){
          builder.append("\t\tGraduate\n");
          builder.append(seperator);             
            for(int i = 0; i < graduateList.size(); i++){
                builder.append(graduateList.get(i) + "\n");               
            }
            return builder.toString(); 
        }
        else{
            builder.append("Invalid Selection");
            return builder.toString();            
        }
    }
    
    private ArrayList <Undergraduate> undergraduateList;
    private ArrayList <Graduate> graduateList;
    private ArrayList <Faculty> facultyList;
    private ArrayList <Staff> staffList;
    private HashMap <Integer, String> idCategoryLookup;
    private HashMap <String, String> nameCategoryLookup;
}
