/*
 * Library.java
 *
 * Created on October 11, 2004, 1:46 AM
 */

package university;

import java.util.*;
import java.io.Serializable;

/**
 * Singelton Class for the library of the university.
 * @author  Nathan Balon
 * University Application
 * CIS 550 - Object Oriented Programming
 * U 0f M Dearborn
 * November 7, 2004
 * @version 1.0
 */
public class Library extends Unit implements Serializable{
    
    /** Creates a new instance of Library */
    private Library() {
        books = new LinkedList<Book>(); 
    }
    
    /** 
      * Creates a new instance of Library 
      * @param name The name of the Library.
      * @param address The Address of the Library.
      */    
    private Library(String name, Address address){
        super(name, address);
        books = new LinkedList<Book>();        
    }
    /**
     * Retruns an instance of Library.
     * @return Unit an instance of Library.
     */
    public static Unit getInstance(){
        return lib;
    }
  
    /**
     * Creates a Libarary unit for the university.
     * @param name The name of the Library.
     * @param address The address of the Library.
     * @return Unit the new unit created.
     */
    public static Unit create(String name, Address address){
        if(lib == null){
            lib = new Library(name, address);
        }
        return lib;
    }    
    
    /**
     * Search if the Library contains a book.
     * @param title The title of the Book to search for.
     * @return boolean if the book exists
     */
    public boolean search(String title){
        for(Iterator<Book> i = books.iterator(); i.hasNext(); ){
            if(i.next().getTitle().equalsIgnoreCase(title)){
                return true;
            }
        }
        return false;
    }
    
    /**
     * Adds a new book to the Library.
     * @param book The book to add to the library.
     */
    public void addBook(Book book){
        books.add(book);
    }
    
    /**
     * Get a list of all the books in the library.
     * @return String a String representing all books in the Library.
     */
    public String getBooks(){
        StringBuffer bookList = new StringBuffer();
        for(Iterator<Book> i = books.iterator(); i.hasNext(); ){  
            bookList.append(i.next().toString());
            bookList.append('\n');
        }
        return bookList.toString();
    }
    
    private static Library lib;
    private LinkedList <Book> books;
}
