/*
 * UnivDirectory.java
 *
 * Created on October 22, 2004, 3:56 AM
 */

package university;

import java.util.*;
import java.io.Serializable;
/**
 * Direcotory of all the units of the university.
 *
 * @author  Nathan Balon
 * University Application
 * CIS 550 - Object Oriented Programming
 * U 0f M Dearborn
 * November 7, 2004
 * @version 1.0
 */
public class UnitDirectory implements Directory, Serializable, Cloneable{
    
    /** Creates a new instance of UnivDirectory */
    public UnitDirectory() {
        educationList = new ArrayList<Person>();
        cisList = new ArrayList<Person>();
        hrList = new ArrayList<Person>();
        addmissionsList = new ArrayList <Person>();
        libraryList = new ArrayList<Person>();
        idUnitLookup = new HashMap<Integer, Unit>();
        nameUnitLookup = new HashMap <String, Unit>();        
    }

    /**
     * Clone a new object
     * @throws java.lang.CloneNotSupportedException Throws CloneNotSupportedException
     * @return Object a cloned object
     */    
    @Override
    public Object clone() throws java.lang.CloneNotSupportedException{
        UnitDirectory cloned = (UnitDirectory)super.clone();
        for(Person person : educationList){
            cloned.educationList.add((Person)person);
        }
        for(Person person : cisList){
            cloned.cisList.add((Person)person);
        }
        for(Person person : hrList){
            cloned.hrList.add((Person)person);
        }        
        for(Person person : addmissionsList){
            cloned.addmissionsList.add((Person)person);
        }
        for(Person person : libraryList){
            cloned.libraryList.add((Person)person);
        }     
        cloned.idUnitLookup = (HashMap)idUnitLookup.clone();
        cloned.nameUnitLookup = (HashMap)nameUnitLookup.clone();
        return cloned;
    }    
        
    /**
     *  Add a Person to the UniversityDirectory 
     *  @param person The person to add to the UniversityDirectory
     */
    public void add(Person person){
        Unit unit = person.getAffiliation();
        String name = person.getName();
        int id = person.getID();
        
        idUnitLookup.put(id, unit);
        nameUnitLookup.put(name, unit);
        
        if(unit instanceof CIS){
            cisList.add(person);
        }
        else if(unit instanceof Library){
            libraryList.add(person);
        }
        else if(unit instanceof HR){
            hrList.add(person);
        }
        else if(unit instanceof Addmissions){
            addmissionsList.add(person);
        }
        else{
            educationList.add(person);
        }
    }
    

    /** 
     *  Remove a Person from the UnitDirectory
     *  @param person The person to remove from the UnitDirectory
     */
    public void remove(Person person){
        // remove the person from both of the lookup table they are contained 
        // in and remove them from the list they are contained in.
        String name = person.getName();
        int id = person.getID();
        Unit unit = person.getAffiliation();
        // remove the person from the hashmaps
        idUnitLookup.remove(id);
        nameUnitLookup.remove(name);  
        
        // search for the correct list to remove the person from
        if(unit instanceof CIS){ 
            for(int i =  0; i < cisList.size(); i++){
                if(cisList.get(i).getID() == id){
                    cisList.remove(i);
                }
            }            
        }
        else if(unit instanceof Education){ 
            for(int i =  0; i < educationList.size(); i++){
                if(educationList.get(i).getID() == id){
                    educationList.remove(i);
                }
            }            
        }        
        else if(unit instanceof Library){
            for(int i =  0; i < libraryList.size(); i++){
                if(libraryList.get(i).getID() == id){
                    libraryList.remove(i);
                }
            }            
        } 
        else if(unit instanceof Addmissions){
            for(int i =  0; i < addmissionsList.size(); i++){
                if(addmissionsList.get(i).getID() == id){
                    addmissionsList.remove(i);
                }
            }            
        }
        else{
            for(int i =  0; i < hrList.size(); i++){
                if(hrList.get(i).getID() == id){
                    hrList.remove(i);
                }
            }            
        }     
       
    }
    /** List all people in the directory 
     *  String Listing all members of the university.
     */
    public String listAll(){
        StringBuilder builder = new StringBuilder();
        // add all the people from each arrayList to list of people
        for(int i = 0; i < libraryList.size() ; i++){
            builder.append(libraryList.get(i) + "\n");
        }
        for(int i = 0; i < hrList.size() ; i++){
            builder.append(hrList.get(i) + "\n");        
        }            
        for(int i = 0; i < addmissionsList.size() ; i++){
            builder.append(addmissionsList.get(i) + "\n");
        } 
        for(int i = 0; i < cisList.size() ; i++){
            builder.append(cisList.get(i) + "\n");
        } 
        for(int i = 0; i < educationList.size() ; i++){
            builder.append(educationList.get(i) + "\n");            
        }    
        return builder.toString();
    }
    
    /**
     * Search for a Person by name.
     * @param name the name of the Person
     * @return the person
     */
    public Person search(String name){
        Unit u = nameUnitLookup.get(name);
        if(u instanceof CIS){
            for(Person p: cisList){
                if(p.getName().equals(name)){
                    return p;
                }
            }
        }
        else if(u instanceof Education){
            for(Person p: educationList){
                if(p.getName().equals(name)){
                    return p;
                }
            }
        } 
        else if(u instanceof Addmissions){
            for(Person p: addmissionsList){
                if(p.getName().equals(name)){
                    return p;
                }
            }
        }
        else if(u instanceof HR){
            for(Person p: hrList){
                if(p.getName().equals(name)){
                    return p;
                }
            }
        }
        else {
            for(Person p: hrList){
                if(p.getName().equals(name)){
                    return p;
                }
            }            
        }
        return null;
    }
    /**
     * Search for a Person by ID.
     * @param id the ID number of the Person
     * @return the person
     */    
    public Person search(int id){
        Unit u = idUnitLookup.get(id);
        if(u instanceof CIS){
            for(Person p: cisList){
                if(p.getID() == id){
                    return p;
                }
            }
        }
        else if(u instanceof Education){
            for(Person p: educationList){
                if(p.getID() == id){
                    return p;
                }
            }
        }
        else if(u instanceof Addmissions){
            for(Person p: addmissionsList){
                if(p.getID() == id){
                    return p;
                }
            }
        } 
        else if(u instanceof HR){
            for(Person p: hrList){
                if(p.getID() == id){
                    return p;
                }
            }
        }
        else{
            for(Person p: libraryList){
                if(p.getID() == id){
                    return p;
                }
            }
        } 
        return null;
    }
    /**
     * Displays all people of a certain category:
     * 1 - cis, 2 - education, 3 - library, 4 - hr, 5 - addmission.
     * categorySelector the category to display.
     */
    public String displayUnit(int categorySelection){
        String  seperator= "-----------------------------------------------"+ 
                           "-----------------------------------------------\n";
        StringBuilder builder = new StringBuilder();
            if(cisList.size() == 0){
                 return new String("No people belong to that unit"); 
            } 
            if(categorySelection == 1){
            builder.append("\t\tCIS\n");
            builder.append(seperator);
            for(int i = 0; i < cisList.size(); i++){
                builder.append(cisList.get(i) + "\n");              
            }
            return builder.toString();              
        }
        else if(categorySelection == 2){
            if(educationList.size() == 0){
                return new String("No people belong to that unit"); 
            }             
            builder.append("\t\tEducation\n");
            builder.append(seperator);
            for(int i = 0; i < educationList.size(); i++){
                builder.append(educationList.get(i) + "\n");           
            }
            return builder.toString();     
        }
        else if(categorySelection == 3){
            if(libraryList.size() == 0){
              return new String("No people belong to that unit"); 
            }             
            builder.append("\t\tLibrary\n");
            builder.append(seperator);            
            for(int i = 0; i < libraryList.size(); i++){
                builder.append(libraryList.get(i) + "\n");                
            }
            return builder.toString();
        } 
        else if(categorySelection == 4){            
          if(hrList.size() == 0){
              return new String("No people belong to that unit"); 
          } 
          builder.append("\t\tHuman Resource\n");
          builder.append(seperator);             
            for(int i = 0; i < hrList.size(); i++){
                builder.append(hrList.get(i) + "\n");               
            }
            return builder.toString(); 
        }
        else if(categorySelection == 5){
          if(addmissionsList.size() == 0){
              return new String("No people belong to that unit"); 
          }
          builder.append("\t\tAddmissions\n");
          builder.append(seperator);             
            for(int i = 0; i < addmissionsList.size(); i++){
                builder.append(addmissionsList.get(i) + "\n");               
            }
            return builder.toString(); 
        }        
        else{
            builder.append("Invalid Selection");
            return builder.toString();            
        }
    }    

    /** Lists the how many people are in each unit.
     *  String A listing the number of people affiliated with each unit.
     */
    public String size(){
        return ("Library:                   " + libraryList.size() +
                "\nHuman Resource:            " + hrList.size() +
                "\nAddmissions:               " + addmissionsList.size() +
                "\nComputer Science:          " + cisList.size() +
                "\nEducation:                 " + educationList.size() + "\n");
    }
    
    // the arrayList contain the people associated with each unit.
    private ArrayList <Person> libraryList;        
    private ArrayList <Person> hrList;
    private ArrayList <Person> addmissionsList;
    private ArrayList <Person> cisList;
    private ArrayList <Person> educationList;
    
    // hash map to look up which unit a person belongs to.
    private HashMap <Integer, Unit> idUnitLookup;
    private HashMap <String, Unit> nameUnitLookup;
}
